import json
from datetime import datetime, timedelta

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "http://lite.tameside.gov.uk/BinCollections/CollectionService.svc/GetBinCollection"
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        params = {
            "version": "3.1.4",
            "uprn": uprn,
            "token": "",
            "notification": "1",
            "operatingsystemid": "2",
            "testmode": "true",
        }

        headers = {"content-type": "text/plain"}

        requests.packages.urllib3.disable_warnings()
        response = requests.post(api_url, json=params, headers=headers)

        json_response = json.loads(response.content)["GetBinCollectionResult"]["Data"]

        today = datetime.today()
        eight_weeks = datetime.today() + timedelta(days=8 * 7)
        data = {"bins": []}
        collection_tuple = []

        bin_friendly_names = {
            "2": "Blue Bin",
            "6": "Green Bin",
            "5": "Black Bin",
            "3": "Brown Bin",
        }

        for item in json_response:
            collection_date = datetime.strptime(
                item.get("CollectionDate"), "%d/%m/%Y %H:%M:%S"
            )
            if today.date() <= collection_date.date() <= eight_weeks.date():
                bin_type = bin_friendly_names.get(item.get("BinType"))
                collection_tuple.append(
                    (bin_type, collection_date.strftime(date_format))
                )

        ordered_data = sorted(collection_tuple, key=lambda x: x[1])

        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1],
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        check_paon(user_paon)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver(web_driver)
        driver.get("https://www.westlothian.gov.uk/article/31528/Bin-Collection-Calendar-Dates")

        # Close feedback banner
        feedbackBanner = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, ".feedback__link--no"))
        )
        feedbackBanner.click()

        # Wait for the postcode field to appear then populate it
        inputElement_postcode = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located(
                (By.ID, "WLBINCOLLECTION_PAGE1_ADDRESSLOOKUPPOSTCODE"))
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        findAddress = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "WLBINCOLLECTION_PAGE1_ADDRESSLOOKUPSEARCH"))
        )
        findAddress.click()

        # Wait for the 'Select address' dropdown to appear and select option matching the house name/number
        WebDriverWait(driver, 10).until(EC.element_to_be_clickable((
            By.XPATH,
            "//select[@id='WLBINCOLLECTION_PAGE1_ADDRESSLOOKUPADDRESS']//option[contains(., '" + user_paon + "')]"
        ))).click()

        # Wait for the collections table to appear
        WebDriverWait(driver, 10).until(EC.presence_of_element_located((By.CSS_SELECTOR, ".bin-collections")))

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Quit Selenium webdriver to release session
        driver.quit()

        # Get collections
        for collection in soup.find_all("div", {"class": "bin-collect"}):
            dict_data = {
                "type": collection.find("h3").get_text(strip=True),
                "collectionDate": datetime.strptime(
                    remove_ordinal_indicator_from_date_string(
                        collection.find("span", {"class": "bin-collect__date"}).get_text(strip=True)
                    ),
                    "%A, %B %d %Y"
                ).strftime(date_format)
            }
            data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return data



import json

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        api_url = f"https://online.bcpcouncil.gov.uk/bcp-apis/?api=BinDayLookup&uprn={user_uprn}"

        requests.packages.urllib3.disable_warnings()
        response = requests.get(api_url)
        json_data = json.loads(response.text)
        data = {"bins": []}
        collections = []

        for bin in json_data:
            bin_type = bin["BinType"]
            next_date = datetime.strptime(bin["Next"], "%m/%d/%Y %H:%M:%S %p")
            subseq_date = datetime.strptime(bin["Subsequent"], "%m/%d/%Y %H:%M:%S %p")
            collections.append((bin_type, next_date))
            collections.append((bin_type, subseq_date))

        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Create the form data
        form_data = {
            "postcode": user_postcode,
            "email-address": "",
            "uprn": user_uprn,
            "gdprTerms": "Yes",
            "privacynoticeid": "323",
            "find": "Show me my collection days",
        }

        # Make a request to the API
        requests.packages.urllib3.disable_warnings()
        s = requests.Session()
        s.get(
            "https://www.calderdale.gov.uk/environment/waste/household-collections/collectiondayfinder.jsp",
            headers={
                "Referer": "https://www.calderdale.gov.uk/environment/waste/household-collections/collectiondayfinder.jsp",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36"
            }
        )
        response = s.post(
            "https://www.calderdale.gov.uk/environment/waste/household-collections/collectiondayfinder.jsp",
            data=form_data,
            headers={
                "Content-Type": "application/x-www-form-urlencoded",
                "Referer": "https://www.calderdale.gov.uk/environment/waste/household-collections/collectiondayfinder.jsp",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36"
            }
        )

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get collections
        row_index = 0
        for row in soup.find("table", {"id": "collection"}).find_all("tr"):
            # Skip headers row
            if row_index < 1:
                row_index += 1
                continue
            else:
                # Get bin info
                bin_info = row.find_all("td")
                # Get the bin type
                bin_type = bin_info[0].find("strong").get_text(strip=True)
                # Get the collection date
                collection_date = ""
                for p in bin_info[2].find_all("p"):
                    if "your next collection" in p.get_text(strip=True):
                        collection_date = datetime.strptime(
                            " ".join(p.get_text(strip=True).replace("will be your next collection.", "").split()),
                            "%A %d %B %Y"
                        )

                if collection_date != "":
                    # Append the bin type and date to the data dict
                    dict_data = {
                        "type": bin_type,
                        "collectionDate": collection_date.strftime(date_format)
                    }
                    data["bins"].append(dict_data)

                row_index += 1

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}
        for bins in soup.select("div[class^=waste-type-container]"):
            bin_type = bins.div.h3.text.strip()
            collection_date = bins.select("div > p")[0].get_text(strip=True)
            next_collection_date = bins.select("div > p")[1].get_text(strip=True)
            if collection_date:
                # Add collection date
                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(collection_date, "%d %B %Y").strftime(date_format)
                }
                data["bins"].append(dict_data)
                # Add next collection date
                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(next_collection_date, "%d %B %Y").strftime(date_format)
                }
                data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



import json

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        headers = {
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Accept-Language": "en-GB,en;q=0.7",
            "Connection": "keep-alive",
            "Content-Type": "application/json; charset=utf-8",
            "Referer": "https://www.fenland.gov.uk/article/13114/?uprn=200002981143&lat=52.665569590474&lng=0.177905443639&postcode=PE13+3SL&line1=20+Felsted+Avenue&rad=5m&layers=2%2C3%2C1",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "Sec-GPC": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
            "X-Requested-With": "XMLHttpRequest",
        }

        # It needs lat and lng for point data, but we don't need it >:)
        params = {
            "type": "loadlayer",
            "layerId": "2",
            "uprn": user_uprn,
            "lat": "0.000000000001",
            "lng": "0.000000000001",
        }

        requests.packages.urllib3.disable_warnings()
        response = requests.get(
            "https://www.fenland.gov.uk/article/13114/", params=params, headers=headers
        )

        # Returned data is just json, so we can get what we need
        json_data = json.loads(response.text)["features"][0]["properties"]["upcoming"]
        data = {"bins": []}

        for item in json_data:
            collections_list = item["collections"]
            for bin in collections_list:
                bin_type = bin["desc"]
                bin_date = datetime.strptime(
                    bin["collectionDate"], "%Y-%m-%dT%H:%M:%SZ"
                ).strftime(date_format)
                dict_data = {
                    "type": bin_type,
                    "collectionDate": bin_date,
                }
                data["bins"].append(dict_data)

        return data



from datetime import date, datetime

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # pindex isn't actually paon, it's a url parameter that I'm guessing the council use as a property id
        data = {"bins": []}
        pindex = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        # WBC use a url parameter called "Track" that's generated when you start a form session.
        # So first off, open the page, find the page link and copy it with the Track
        start_url = "https://wav-wrp.whitespacews.com/"
        s = requests.session()
        response = s.get(start_url)
        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()
        collection_page_link = soup.find_all(
            "p", {"class": "govuk-body govuk-!-margin-bottom-0 colorblue lineheight15"}
        )[0].find("a")["href"]
        track_id = collection_page_link[33:60]

        # Next we need to search using the postcode, but this is actually an important POST request
        pc_headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,"
            "image/webp,image/apng,*/*;q=0.8",
            "Accept-Language": "en-GB,en;q=0.9",
            "Cache-Control": "max-age=0",
            "Connection": "keep-alive",
            "Origin": "https://wav-wrp.whitespacews.com",
            "Referer": "https://wav-wrp.whitespacews.com/"
            + track_id
            + "&serviceID=A&seq=2",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Sec-GPC": "1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, "
            "like Gecko) Chrome/106.0.0.0 Safari/537.36",
        }
        form_data = {
            "address_name_number": "",
            "address_street": "",
            "street_town": "",
            "address_postcode": user_postcode,
        }
        response = s.post(
            "https://wav-wrp.whitespacews.com/mop.php?serviceID=A&"
            + track_id
            + "&seq=2",
            headers=pc_headers,
            data=form_data,
        )

        # Finally, we can use pindex to find the address and get some data
        request_url = (
            "https://wav-wrp.whitespacews.com/mop.php?"
            + track_id
            + "&serviceID=A&seq=3&pIndex="
            + pindex
        )
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,"
            "image/webp,image/apng,*/*;q=0.8",
            "Accept-Language": "en-GB,en;q=0.9",
            "Connection": "keep-alive",
            "Referer": "https://wav-wrp.whitespacews.com/mop.php?serviceID=A&"
            + track_id
            + "&seq=2",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Sec-GPC": "1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, "
            "like Gecko) Chrome/106.0.0.0 Safari/537.36",
        }

        response = s.get(request_url, headers=headers)
        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()

        # Find the list elements
        u1_block = soup.find_all(
            "u1",
            {
                "class": "displayinlineblock justifycontentleft alignitemscenter margin0 padding0"
            },
        )

        for element in u1_block:
            x = element.find_all_next(
                "li", {"class": "displayinlineblock padding0px20px5px0px"}
            )
            dict_data = {
                "type": x[2].text.strip(),
                "collectionDate": datetime.strptime(
                    x[1].text.strip(), date_format
                ).strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



"""Module that contains an abstract class that can be imported to
handle the data recieved from the provided council class.

Keyword arguments: None
"""
import json
import logging
from abc import ABC, abstractmethod
from logging.config import dictConfig

import requests

from uk_bin_collection.uk_bin_collection.common import update_input_json

LOGGING_CONFIG = dict(
    version=1,
    formatters={"f": {"format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s"}},
    handlers={
        "h": {"class": "logging.StreamHandler", "formatter": "f", "level": logging.INFO}
    },
    root={"handlers": ["h"], "level": logging.INFO},
)


def setup_logging(logging_config, logger_name):
    try:
        logging.config.dictConfig(logging_config)
        logger = logging.getLogger(logger_name)
        return logger
    except Exception as exp:
        raise exp


LOGGER = setup_logging(LOGGING_CONFIG, None)


# import the wonderful Beautiful Soup and the URL grabber


class AbstractGetBinDataClass(ABC):
    """An abstract class that can be imported to handle the data recieved from the provided
    council class.

    Keyword arguments: None
    """

    def template_method(self, address_url: str, **kwargs) -> None:  # pragma: no cover
        """The main template method that is constructed

        Keyword arguments:
        address_url -- the url to get the data from
        """
        this_url = address_url
        this_postcode = kwargs.get("postcode", None)
        this_paon = kwargs.get("paon", None)
        this_uprn = kwargs.get("uprn", None)
        this_usrn = kwargs.get("usrn", None)
        this_web_driver = kwargs.get("web_driver", None)
        skip_get_url = kwargs.get("skip_get_url", None)
        dev_mode = kwargs.get("dev_mode", False)
        council_module_str = kwargs.get("council_module_str", None)
        if (
                not skip_get_url or skip_get_url is False
        ):  # we will not use the generic way to get data - needs a get data in the council class itself
            page = self.get_data(address_url)
            bin_data_dict = self.parse_data(
                page, postcode=this_postcode, paon=this_paon, uprn=this_uprn, usrn=this_usrn, web_driver=this_web_driver, url=this_url
            )
            json_output = self.output_json(bin_data_dict)
        else:
            bin_data_dict = self.parse_data(
                "", postcode=this_postcode, paon=this_paon, uprn=this_uprn, usrn=this_usrn, web_driver=this_web_driver, url=this_url
            )
            json_output = self.output_json(bin_data_dict)

        # if dev mode create/update council's entry in the input.json
        if dev_mode is not None and dev_mode is True:
            update_input_json(council_module_str, this_url, postcode=this_postcode, paon=this_paon, uprn=this_uprn, usrn=this_usrn, web_driver=this_web_driver, skip_get_url=skip_get_url)

        return json_output

    @classmethod
    def get_data(cls, url) -> str:
        """This method makes the request to the council

        Keyword arguments:
        url -- the url to get the data from
        """
        # Set a user agent so we look like a browser ;-)
        user_agent = (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/108.0.0.0 Safari/537.36"
        )
        headers = {"User-Agent": user_agent}
        requests.packages.urllib3.disable_warnings()

        # Make the Request - change the URL - find out your property number
        try:
            full_page = requests.get(url, headers, verify=False)
            return full_page
        except requests.exceptions.HTTPError as errh:
            LOGGER.error(f"Http Error: {errh}")
            raise
        except requests.exceptions.ConnectionError as errc:
            LOGGER.error(f"Error Connecting: {errc}")
            raise
        except requests.exceptions.Timeout as errt:
            LOGGER.error(f"Timeout Error: {errt}")
            raise
        except requests.exceptions.RequestException as err:
            LOGGER.error(f"Oops: Something Else {err}")
            raise

    @abstractmethod
    def parse_data(self, page: str, **kwargs) -> dict:
        """Abstract method that takes a page as a string

        Keyword arguments:
        page -- a string from the requested page
        """

    @classmethod
    def output_json(cls, bin_data_dict: dict) -> str:
        """Method to output the json as a pretty printed string

        Keyword arguments:
        bin_data_dict -- a dict parsed data
        """
        # Form a JSON wrapper
        # Make the JSON

        json_data = json.dumps(bin_data_dict, sort_keys=False, indent=4)

        # Output the data
        return json_data



import json
from datetime import datetime

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = (
            "https://waste-api.york.gov.uk/api/Collections/GetBinCollectionDataForUprn/"
        )
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        requests.packages.urllib3.disable_warnings()
        response = requests.get(f"{api_url}{uprn}")
        json_response = json.loads(response.content)["services"]
        data = {"bins": []}
        collection_tuple = []

        for item in json_response:
            collection_date = datetime.strptime(
                item.get("nextCollection"), "%Y-%m-%dT%H:%M:%S"
            ).strftime(date_format)
            collection_tuple.append((item.get("service"), collection_date))

        ordered_data = sorted(collection_tuple, key=lambda x: x[1])

        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1],
            }
            data["bins"].append(dict_data)

        return data



import logging
import pytest
import traceback
from pytest_bdd import scenario, given, when, then, parsers
from hamcrest import assert_that, equal_to

from step_helpers import file_handler
from uk_bin_collection.uk_bin_collection import collect_data

logging.basicConfig(level=logging.INFO, format='%(levelname)s - %(message)s')

@scenario("../features/validate_council_outputs.feature", "Validate Council Output")
def test_scenario_outline():
    pass


@pytest.fixture
def context():
    class Context(object):
        pass

    return Context()


@given(parsers.parse("the council: {council_name}"))
def get_council_step(context, council_name):
    try:
        council_input_data = file_handler.load_inputs_file("input.json")
        context.metadata = council_input_data[council_name]
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Validate Output: {err}")
        raise (err)

# When we scrape the data from <council> using <selenium_mode> and the <selenium_url> is set.
@when(parsers.parse("we scrape the data from {council} using {selenium_mode} and the {selenium_url} is set"))
def scrape_step(context, council, selenium_mode, selenium_url):
    context.council = council
    context.selenium_mode = selenium_mode
    context.selenium_url = selenium_url

    args = [council, context.metadata["url"]]

    if "uprn" in context.metadata:
        uprn = context.metadata["uprn"]
        args.append(f"-u={uprn}")
    if "postcode" in context.metadata:
        postcode = context.metadata["postcode"]
        args.append(f"-p={postcode}")
    if "house_number" in context.metadata:
        house_number = context.metadata["house_number"]
        args.append(f"-n={house_number}")
    if "usrn" in context.metadata:
        usrn = context.metadata["usrn"]
        args.append(f"-us={usrn}")
    # TODO we should somehow run this test with and without this argument passed
    # TODO I do think this would make the testing of the councils a lot longer and cause a double hit from us

    # At the moment the feature file is set to local execution of the selenium so no url will be set
    # And it the behave test will execute locally
    if selenium_mode != 'None' and selenium_url != 'None':
        if selenium_mode != 'local': 
            web_driver = context.metadata["web_driver"]
            args.append(f"-w={web_driver}")
    if "skip_get_url" in context.metadata:
        args.append(f"-s")

    try:
        CollectData = collect_data.UKBinCollectionApp()
        CollectData.set_args(args)
        context.parse_result = CollectData.run()
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Schema: {err}")
        raise (err)


@then("the result is valid json")
def validate_json_step(context):
    try:
        valid_json = file_handler.validate_json(context.parse_result)
        assert_that(valid_json, True)
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Validate Output: {err}")
        logging.info(f"JSON Output: {context.parse_result}")
        raise (err)


@then("the output should validate against the schema")
def validate_output_step(context):
    try:
        council_schema = file_handler.load_schema_file(f"output.schema")
        schema_result = file_handler.validate_json_schema(
            context.parse_result, council_schema
        )
        assert_that(schema_result, True)
    except Exception as err:
        logging.error(traceback.format_exc())
        logging.info(f"Validate Output: {err}")
        logging.info(f"JSON Output: {context.parse_result}")
        raise (err)



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://webforms.rochdale.gov.uk/BinCalendar"
        user_postcode = kwargs.get("postcode")
        user_uprn = kwargs.get("uprn")

        # Check the postcode and UPRN are valid
        check_postcode(user_postcode)
        check_uprn(user_uprn)

        # Create the form data
        form_data = {
            "PostCode": user_postcode,
            "SelectedUprn": user_uprn,
            "Step": 2,
        }

        # Make a request to the API
        requests.packages.urllib3.disable_warnings()
        response = requests.post(api_url, data=form_data)

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get the table element and rows
        table_element = soup.find("table", {"id": "tblCollectionDetails"})
        table_rows = table_element.find_all_next("tr")

        row_index = 0
        for row in table_rows:
            if row_index < 1:
                row_index += 1
                continue
            else:
                # Get the date from the th element
                date = datetime.strptime(row.find("th").get_text().strip(), "%A %d %B %Y").strftime(date_format)

                # Get the bin types from the td elements and filter out the empty ones
                bin_types = filter(lambda td: td.find("img"), row.find_all("td"))

                # Convert the bin types to a list
                bin_types_list = list(bin_types)

                # Append the bin type and date to the data dict
                for td in bin_types_list:
                    img = td.find("img")
                    bin_type_text = img["alt"]
                    data["bins"].append({"type": bin_type_text, "collectionDate": date})

                row_index += 1

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        data = {"bins": []}

        # Start a new session
        requests.packages.urllib3.disable_warnings()
        s = requests.session()

        headers = {
            'authority': 'www.bolton.gov.uk',
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'accept-language': 'en-GB,en;q=0.9',
            'cache-control': 'no-cache',
            'content-type': 'application/x-www-form-urlencoded',
            'origin': 'https://www.bolton.gov.uk',
            'pragma': 'no-cache',
            'referer': 'https://www.bolton.gov.uk/next-bin-collection',
            'sec-ch-ua': '"Not?A_Brand";v="99", "Opera GX";v="97", "Chromium";v="111"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest': 'document',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-user': '?1',
            'upgrade-insecure-requests': '1',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.5563.147 Safari/537.36',
        }
        
        # Get our initial session running
        response = s.get("https://carehomes.bolton.gov.uk/bins.aspx", headers=headers)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Grab the variables needed to continue
        payload = {
            "__VIEWSTATE": (
                soup.find("input", {"id": "__VIEWSTATE"}).get("value")
            ),
            "__VIEWSTATEGENERATOR": (
                soup.find("input", {"id": "__VIEWSTATEGENERATOR"}).get("value")
            ),
            "__EVENTVALIDATION": (
                soup.find("input", {"id": "__EVENTVALIDATION"}).get("value")
            ),
            "txtPostcode": (user_postcode),
            "btnSubmit": "Submit"
        }

        # Get the address selection page
        response = s.post("https://carehomes.bolton.gov.uk/bins.aspx", data=payload, headers=headers)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Grab the variables needed to continue
        payload = {
            "__VIEWSTATE": (
                soup.find("input", {"id": "__VIEWSTATE"}).get("value")
            ),
            "__VIEWSTATEGENERATOR": (
                soup.find("input", {"id": "__VIEWSTATEGENERATOR"}).get("value")
            ),
            "__EVENTVALIDATION": (
                soup.find("input", {"id": "__EVENTVALIDATION"}).get("value")
            ),
            "txtPostcode": (user_postcode),
            "ddlAddresses": (user_uprn)
        }

        # Get the final page with the actual bin data
        response = s.post("https://carehomes.bolton.gov.uk/bins.aspx", data=payload, headers=headers)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        collections = []

        # Find section with bins in
        sections = soup.find_all("div", {"class": "bin-info"})

        # For each bin section, get the text and the list elements
        for item in sections:
            words = item.find_next("strong").text.split()[2:4]
            bin_type = ' '.join(words).capitalize()
            date_list = item.find_all("p")
            for d in date_list:
                next_collection = datetime.strptime(d.text.strip(), "%A %d %B %Y")
                collections.append((bin_type, next_collection))

        # Sort the text and list elements by date
        ordered_data = sorted(collections, key=lambda x: x[1])

        # Put the elements into the dictionary
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data




from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        # Get the paragraph lines from the page
        data = {"bins": []}
        page_text = soup.find("div", {"class": "collect"}).find_all("p")

        # Parse the correct lines (find them, remove the ordinal indicator and make them the correct format date) and
        # then add them to the dictionary
        rubbish_day = datetime.strptime(remove_ordinal_indicator_from_date_string(page_text[2].find_next("strong").text), "%d %B %Y").strftime(date_format)
        dict_data = {
            "type": "Rubbish",
            "collectionDate": rubbish_day,
        }
        data["bins"].append(dict_data)
        recycling_day = datetime.strptime(remove_ordinal_indicator_from_date_string(page_text[4].find_next("strong").text), "%d %B %Y").strftime(date_format)
        dict_data = {
            "type": "Recycling",
            "collectionDate": recycling_day,
        }
        data["bins"].append(dict_data)


        return data



from xml.etree import ElementTree

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    baseclass. They can also override some
    operations with a default implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        check_uprn(uprn)
        council = "SNO"

        # Make SOAP request
        headers = {
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Accept-Language": "en-GB,en-US;q=0.9,en;q=0.8",
            "Connection": "keep-alive",
            "Content-Type": "text/xml; charset=UTF-8",
            "Host": "collections-southnorfolk.azurewebsites.net",
            "Origin": "https://collections-southnorfolk.azurewebsites.net",
            "Referer": "https://collections-southnorfolk.azurewebsites.net/calendar.html",
            "sec-ch-ua": '"Chromium";v="110", "Not A(Brand";v="24", "Google Chrome";v="110"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36",
        }
        requests.packages.urllib3.disable_warnings()
        post_data = (
                '<?xml version="1.0" encoding="utf-8"?>'
                '<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">'
                '<soap:Body><getRoundCalendarForUPRN xmlns="http://webaspx-collections.azurewebsites.net/">'
                "<council>" + council + "</council><UPRN>" + uprn + "</UPRN>"
                                                                    "<from>Chtml</from></getRoundCalendarForUPRN></soap:Body></soap:Envelope>"
        )
        response = requests.post(
            "https://collections-southnorfolk.azurewebsites.net/WSCollExternal.asmx",
            headers=headers,
            data=post_data,
        )
        if response.status_code != 200:
            raise ValueError("No bin data found for provided UPRN.")

        # Get HTML from SOAP response
        xmltree = ElementTree.fromstring(response.text)
        html = xmltree.find(
            ".//{http://webaspx-collections.azurewebsites.net/}getRoundCalendarForUPRNResult"
        ).text
        # Parse with BS4
        soup = BeautifulSoup(html, features="html.parser")
        soup.prettify()

        data = {"bins": []}
        for bin_type in ["RefuseBin", "RecycleBin", "GardenBin"]:
            bin_el = soup.find("b", string=bin_type)
            if bin_el:
                bin_info = bin_el.next_sibling.split(": ")[1]
                collection_date = ""
                results = re.search("([A-Za-z]+ \\d\\d? [A-Za-z]+) then", bin_info)
                if results:
                    date = get_next_occurrence_from_day_month(datetime.strptime(
                        results[1] + " " + datetime.now().strftime("%Y"), "%a %d %b %Y"
                    ))
                    if date:
                        collection_date = date.strftime(date_format)
                else:
                    results2 = re.search("([A-Za-z]+) then", bin_info)
                    if results2:
                        collection_date = results2[1]

                if collection_date != "":
                    dict_data = {
                        "type": bin_type,
                        "collectionDate": collection_date,
                    }
                    data["bins"].append(dict_data)

        return data



import ast

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        requests.packages.urllib3.disable_warnings()
        s = requests.session()

        service_type_headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,"
            "image/webp,image/apng,*/*;q=0.8",
            "Accept-Language": "en-GB,en;q=0.9",
            "Cache-Control": "max-age=0",
            "Connection": "keep-alive",
            "Referer": "https://www.bristol.gov.uk/",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "cross-site",
            "Sec-Fetch-User": "?1",
            "Sec-GPC": "1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, "
            "like Gecko) Chrome/105.0.0.0 Safari/537.36",
        }
        service_type_params = {
            "servicetypeid": "7dce896c-b3ba-ea11-a812-000d3a7f1cdc",
        }
        response = s.get(
            "https://bristolcouncil.powerappsportals.com/completedynamicformunauth/",
            params=service_type_params,
            headers=service_type_headers,
        )

        llpg_headers = {
            "Accept": "*/*",
            "Accept-Language": "en-GB,en;q=0.9",
            "Connection": "keep-alive",
            "Ocp-Apim-Subscription-Key": "47ffd667d69c4a858f92fc38dc24b150",
            "Ocp-Apim-Trace": "true",
            "Origin": "https://bristolcouncil.powerappsportals.com",
            "Referer": "https://bristolcouncil.powerappsportals.com/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "cross-site",
            "Sec-GPC": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, "
            "like Gecko) Chrome/105.0.0.0 Safari/537.36",
        }
        llpg_uprn = "UPRN" + user_uprn
        llpg_json_data = {
            "Uprn": llpg_uprn,
        }
        response = s.post(
            "https://bcprdapidyna002.azure-api.net/bcprdfundyna001-llpg/DetailedLLPG",
            headers=llpg_headers,
            json=llpg_json_data,
        )

        headers = {
            "Accept": "*/*",
            "Accept-Language": "en-GB,en;q=0.9",
            "Connection": "keep-alive",
            # Already added when you pass json=
            # 'Content-Type': 'application/json',
            "Ocp-Apim-Subscription-Key": "47ffd667d69c4a858f92fc38dc24b150",
            "Ocp-Apim-Trace": "true",
            "Origin": "https://bristolcouncil.powerappsportals.com",
            "Referer": "https://bristolcouncil.powerappsportals.com/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "cross-site",
            "Sec-GPC": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",
        }
        json_data = {
            "uprn": user_uprn,
        }
        response = s.post(
            "https://bcprdapidyna002.azure-api.net/bcprdfundyna001-alloy/NextCollectionDates",
            headers=headers,
            json=json_data,
        )

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Soup returns API response rather than HTML, so parse those strings
        string_data = soup.text.split("data")[1]
        collection_data = string_data.split("]}")

        # Remove the spare ] and , characters at the of each list element
        fixed_data = [i[1:] for i in collection_data]

        # Remove the last list element since it's garbage (funny since this is a bin project)
        fixed_data.pop()
        collection_data.clear()

        # Make some more changes:
        idx = 0
        for i in fixed_data:
            if idx == 0:
                # Remove two extra characters if it's the first element
                i = i[2:]
            # Append some characters to the end of each line to make to dict
            i = i + "]}"
            idx += 1
            # Reuse the collection_data list to make a list of dictionaries - one for each bin
            collection_data.append(ast.literal_eval(i))

        collections = []
        for bin in collection_data:
            bin_type = bin["containerName"]
            next_collection = datetime.strptime(
                bin["collection"][0]["nextCollectionDate"], "%Y-%m-%dT%H:%M:%S"
            ).strftime(date_format)
            # Could work out next date using the roundDescription and the is_holiday function in common.py
            collections.append((bin_type, next_collection))

        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {"type": item[0], "collectionDate": item[1]}
            data["bins"].append(dict_data)

        return data
